@@ -11,6 +11,7 @@ from urllib import parse
 
 import wrapt
 
+import ddtrace
 from ddtrace import config
 from ddtrace._trace._inferred_proxy import create_inferred_proxy_span_if_headers_exist
 from ddtrace._trace._span_pointer import _SpanPointerDescription
@@ -28,6 +29,7 @@ from ddtrace.ext import SpanKind
 from ddtrace.ext import azure_servicebus as azure_servicebusx
 from ddtrace.ext import db
 from ddtrace.ext import http
+from ddtrace.ext import redis as redisx
 from ddtrace.internal import core
 from ddtrace.internal.compat import maybe_stringify
 from ddtrace.internal.constants import COMPONENT
@@ -99,26 +101,27 @@ class _TracedIterable(wrapt.ObjectProxy):
         return super(_TracedIterable, self).__getattribute__(name)
 
 
-def _get_parameters_for_new_span_directly_from_context(ctx: core.ExecutionContext) -> Dict[str, str]:
+def _get_parameters_for_new_span_directly_from_context(ctx: core.ExecutionContext) -> Dict[str, Any]:
     span_kwargs = {}
     for parameter_name in {"span_type", "resource", "service", "child_of", "activate"}:
-        parameter_value = ctx.get_local_item(parameter_name)
+        parameter_value = ctx.get_item(parameter_name)
         if parameter_value:
             span_kwargs[parameter_name] = parameter_value
     return span_kwargs
 
 
 def _start_span(ctx: core.ExecutionContext, call_trace: bool = True, **kwargs) -> "Span":
-    activate_distributed_headers = ctx.get_local_item("activate_distributed_headers")
+    activate_distributed_headers = ctx.get_item("activate_distributed_headers")
     span_kwargs = _get_parameters_for_new_span_directly_from_context(ctx)
     call_trace = ctx.get_item("call_trace", call_trace)
-    tracer = ctx.get_item("tracer") or (ctx.get_item("middleware") or ctx["pin"]).tracer
+    # Look for the tracer in the context, or fallback to the global tracer
+    tracer = ctx.get_item("tracer") or (ctx.get_item("middleware") or ctx.get_item("pin") or ddtrace).tracer
     integration_config = ctx.get_item("integration_config")
     if integration_config and activate_distributed_headers:
         trace_utils.activate_distributed_headers(
             tracer,
             int_config=integration_config,
-            request_headers=ctx["distributed_headers"],
+            request_headers=ctx.get_item("distributed_headers"),
             override=ctx.get_item("distributed_headers_config_override"),
         )
     distributed_context = ctx.get_item("distributed_context")
@@ -132,10 +135,18 @@ def _start_span(ctx: core.ExecutionContext, call_trace: bool = True, **kwargs) -
         span_kwargs = ctx.get_item("span_kwargs", span_kwargs)
 
     span_kwargs.update(kwargs)
-    span = (tracer.trace if call_trace else tracer.start_span)(ctx["span_name"], **span_kwargs)
+    span_name = ctx.get_item("span_name")
+    if not span_name:
+        raise ValueError("span_name must be set in the context before starting a span")
+    span = (tracer.trace if call_trace else tracer.start_span)(span_name, **span_kwargs)
 
-    for tk, tv in ctx.get_item("tags", dict()).items():
-        span.set_tag_str(tk, tv)
+    tags: Optional[Dict[str, str]] = ctx.get_item("tags")
+    if tags:
+        for tk, tv in tags.items():
+            span.set_tag(tk, tv)
+    if ctx.get_item("measured"):
+        # PERF: avoid setting via Span.set_tag
+        span.set_metric(_SPAN_MEASURED_KEY, 1)
 
     ctx.span = span
 
@@ -175,12 +186,20 @@ def _set_web_frameworks_tags(ctx, span, int_config):
 
 def _on_web_framework_start_request(ctx, int_config):
     request_span = ctx.get_item("req_span")
+    if ctx.get_item("allow_default_resource") is True:
+        ctx.set_item("set_resource", True)
     _set_web_frameworks_tags(ctx, request_span, int_config)
 
 
 def _on_web_framework_finish_request(
     span, int_config, method, url, status_code, query, req_headers, res_headers, route, finish, **kwargs
 ):
+    if core.get_item("set_resource", default=False) is True and status_code is not None:
+        try:
+            status_code = int(status_code)
+        except ValueError:
+            pass
+        span.resource = "{} {}".format(method, status_code)
     trace_utils.set_http_meta(
         span=span,
         integration_config=int_config,
@@ -194,6 +213,8 @@ def _on_web_framework_finish_request(
         **kwargs,
     )
     _set_inferred_proxy_tags(span, status_code)
+    for tk, tv in core.get_item("additional_tags", default=dict()).items():
+        span.set_tag_str(tk, tv)
 
     if finish:
         span.finish()
@@ -258,30 +279,30 @@ def _on_inferred_proxy_finish(ctx):
 
 
 def _on_traced_request_context_started_flask(ctx):
-    current_span = ctx["pin"].tracer.current_span()
-    if not ctx["pin"].enabled or not current_span:
+    current_span = ctx.get_item("pin").tracer.current_span()
+    if not ctx.get_item("pin").enabled or not current_span:
         return
 
     ctx.span = current_span
-    flask_config = ctx["flask_config"]
-    _set_flask_request_tags(ctx["flask_request"], current_span, flask_config)
+    flask_config = ctx.get_item("flask_config")
+    _set_flask_request_tags(ctx.get_item("flask_request"), current_span, flask_config)
     request_span = _start_span(ctx)
     request_span._ignore_exception(ctx.get_item("ignored_exception_type"))
 
 
 def _maybe_start_http_response_span(ctx: core.ExecutionContext) -> None:
-    request_span = ctx["request_span"]
-    middleware = ctx["middleware"]
-    status_code, status_msg = ctx["status"].split(" ", 1)
+    request_span = ctx.get_item("request_span")
+    middleware = ctx.get_item("middleware")
+    status_code, status_msg = ctx.get_item("status").split(" ", 1)
     trace_utils.set_http_meta(
-        request_span, middleware._config, status_code=status_code, response_headers=ctx["environ"]
+        request_span, middleware._config, status_code=status_code, response_headers=ctx.get_item("environ")
     )
     if ctx.get_item("start_span", False):
         request_span.set_tag_str(http.STATUS_MSG, status_msg)
         _start_span(
             ctx,
             call_trace=False,
-            child_of=ctx["parent_call"],
+            child_of=ctx.get_item("parent_call"),
             activate=True,
         )
 
@@ -498,7 +519,7 @@ def _on_request_span_modifier_post(ctx, flask_config, request, req_body):
 
 
 def _on_traced_get_response_pre(_, ctx: core.ExecutionContext, request, before_request_tags):
-    before_request_tags(ctx["pin"], ctx.span, request)
+    before_request_tags(ctx.get_item("pin"), ctx.span, request)
     ctx.span._metrics[_SPAN_MEASURED_KEY] = 1
 
 
@@ -512,9 +533,9 @@ def _on_web_request_final_tags(span):
 def _on_django_finalize_response_pre(ctx, after_request_tags, request, response):
     # DEV: Always set these tags, this is where `span.resource` is set
     span = ctx.span
-    after_request_tags(ctx["pin"], span, request, response)
+    after_request_tags(ctx.get_item("pin"), span, request, response)
 
-    trace_utils.set_http_meta(span, ctx["integration_config"], route=span.get_tag("http.route"))
+    trace_utils.set_http_meta(span, ctx.get_item("integration_config"), route=span.get_tag("http.route"))
     _set_inferred_proxy_tags(span, None)
 
 
@@ -527,7 +548,7 @@ def _on_django_start_response(
 
     trace_utils.set_http_meta(
         ctx.span,
-        ctx["integration_config"],
+        ctx.get_item("integration_config"),
         method=request.method,
         query=query,
         raw_uri=uri,
@@ -538,8 +559,16 @@ def _on_django_start_response(
     )
 
 
-def _on_django_cache(ctx: core.ExecutionContext, rowcount: int):
-    ctx.span.set_metric(db.ROWCOUNT, rowcount)
+def _on_django_cache(
+    ctx: core.ExecutionContext,
+    exc_info: Tuple[Optional[type], Optional[BaseException], Optional[TracebackType]],
+) -> None:
+    try:
+        rowcount = ctx.get_item("rowcount")
+        if rowcount is not None:
+            ctx.span.set_metric(db.ROWCOUNT, rowcount)
+    finally:
+        return _finish_span(ctx, exc_info)
 
 
 def _on_django_func_wrapped(_unused1, _unused2, _unused3, ctx, ignored_excs):
@@ -675,11 +704,11 @@ def _on_botocore_patched_stepfunctions_update_input(ctx, span, _, trace_data, __
 
 def _on_botocore_patched_bedrock_api_call_started(ctx, request_params):
     span = ctx.span
-    integration = ctx["bedrock_integration"]
+    integration = ctx.get_item("bedrock_integration")
     integration._tag_proxy_request(ctx)
 
-    span.set_tag_str("bedrock.request.model_provider", ctx["model_provider"])
-    span.set_tag_str("bedrock.request.model", ctx["model_name"])
+    span.set_tag_str("bedrock.request.model_provider", ctx.get_item("model_provider"))
+    span.set_tag_str("bedrock.request.model", ctx.get_item("model_name"))
 
     if "n" in request_params:
         ctx.set_item("num_generations", str(request_params["n"]))
@@ -688,15 +717,15 @@ def _on_botocore_patched_bedrock_api_call_started(ctx, request_params):
 def _on_botocore_patched_bedrock_api_call_exception(ctx, exc_info):
     span = ctx.span
     span.set_exc_info(*exc_info)
-    model_name = ctx["model_name"]
-    integration = ctx["bedrock_integration"]
+    model_name = ctx.get_item("model_name")
+    integration = ctx.get_item("bedrock_integration")
     if "embed" not in model_name:
         integration.llmobs_set_tags(span, args=[ctx], kwargs={})
     span.finish()
 
 
 def _propagate_context(ctx, headers):
-    distributed_tracing_enabled = ctx["integration_config"].distributed_tracing_enabled
+    distributed_tracing_enabled = ctx.get_item("integration_config").distributed_tracing_enabled
     span = ctx.span
     if distributed_tracing_enabled and span:
         HTTPPropagator.inject(span.context, headers)
@@ -717,14 +746,14 @@ def _on_end_of_traced_method_in_fork(ctx):
     """Force flush to agent since the process `os.exit()`s
     immediately after this method returns
     """
-    ctx["pin"].tracer.flush()
+    ctx.get_item("pin").tracer.flush()
 
 
 def _on_botocore_bedrock_process_response_converse(
     ctx: core.ExecutionContext,
     result: List[Dict[str, Any]],
 ):
-    ctx["bedrock_integration"].llmobs_set_tags(
+    ctx.get_item("bedrock_integration").llmobs_set_tags(
         ctx.span,
         args=[ctx],
         kwargs={},
@@ -738,8 +767,8 @@ def _on_botocore_bedrock_process_response(
     formatted_response: Dict[str, Any],
 ) -> None:
     with ctx.span as span:
-        model_name = ctx["model_name"]
-        integration = ctx["bedrock_integration"]
+        model_name = ctx.get_item("model_name")
+        integration = ctx.get_item("bedrock_integration")
         if "embed" in model_name:
             return
         integration.llmobs_set_tags(span, args=[ctx], kwargs={}, response=formatted_response)
@@ -775,6 +804,18 @@ def _on_redis_command_post(ctx: core.ExecutionContext, rowcount):
         ctx.span.set_metric(db.ROWCOUNT, rowcount)
 
 
+def _on_redis_execute_pipeline(ctx: core.ExecutionContext, pin, config_integration, args, instance, query):
+    span = ctx.span
+    if args is not None:
+        # PERF: avoid extra overhead from checks in Span.set_metric
+        span._metrics[redisx.ARGS_LEN] = len(args)
+    else:
+        for attr in ("command_stack", "_command_stack"):
+            if hasattr(instance, attr):
+                # PERF: avoid extra overhead from checks in Span.set_metric
+                span._metrics[redisx.PIPELINE_LEN] = len(getattr(instance, attr))
+
+
 def _on_valkey_command_post(ctx: core.ExecutionContext, rowcount):
     if rowcount is not None:
         ctx.span.set_metric(db.ROWCOUNT, rowcount)
@@ -869,6 +910,22 @@ def _on_azure_servicebus_send_message_modifier(ctx, azure_servicebus_config, ent
     span.set_tag_str(SPAN_KIND, SpanKind.PRODUCER)
 
 
+def _on_router_match(route):
+    req_span = core.get_item("req_span")
+    core.set_item("set_resource", False)
+    req_span.resource = "{} {}".format(
+        route.method,
+        route.template,
+    )
+
+    MOLTEN_ROUTE = "molten.route"
+
+    if not req_span.get_tag(MOLTEN_ROUTE):
+        req_span.set_tag_str(MOLTEN_ROUTE, route.name)
+    if not req_span.get_tag(http.ROUTE):
+        req_span.set_tag_str(http.ROUTE, route.template)
+
+
 def listen():
     core.on("wsgi.request.prepare", _on_request_prepare)
     core.on("wsgi.request.prepared", _on_request_prepared)
@@ -885,7 +942,6 @@ def listen():
     core.on("django.traced_get_response.pre", _on_traced_get_response_pre)
     core.on("django.finalize_response.pre", _on_django_finalize_response_pre)
     core.on("django.start_response", _on_django_start_response)
-    core.on("django.cache", _on_django_cache)
     core.on("django.func.wrapped", _on_django_func_wrapped)
     core.on("django.block_request_callback", _on_django_block_request)
     core.on("django.after_request_headers.post", _on_django_after_request_headers_post)
@@ -916,6 +972,7 @@ def listen():
     core.on("botocore.kinesis.GetRecords.post", _on_botocore_kinesis_getrecords_post)
     core.on("redis.async_command.post", _on_redis_command_post)
     core.on("redis.command.post", _on_redis_command_post)
+    core.on("redis.execute_pipeline", _on_redis_execute_pipeline)
     core.on("valkey.async_command.post", _on_valkey_command_post)
     core.on("valkey.command.post", _on_valkey_command_post)
     core.on("azure.functions.request_call_modifier", _on_azure_functions_request_span_modifier)
@@ -939,6 +996,7 @@ def listen():
     core.on("rq.worker.perform_job", _after_job_execution)
     core.on("rq.worker.after.perform.job", _on_end_of_traced_method_in_fork)
     core.on("rq.queue.enqueue_job", _propagate_context)
+    core.on("molten.router.match", _on_router_match)
 
     for context_name in (
         # web frameworks
@@ -947,6 +1005,7 @@ def listen():
         "cherrypy.request",
         "falcon.request",
         "molten.request",
+        "molten.trace_func",
         "pyramid.request",
         "sanic.request",
         "tornado.request",
@@ -961,6 +1020,13 @@ def listen():
         "wsgi.__call__",
         "django.traced_get_response",
         "django.cache",
+        "django.middleware.__call__",
+        "django.middleware.func",
+        "django.middleware.process_exception",
+        "django.middleware.process_request",
+        "django.middleware.process_response",
+        "django.middleware.process_template_response",
+        "django.middleware.process_view",
         "django.template.render",
         "django.process_exception",
         "django.func.wrapped",
@@ -972,6 +1038,7 @@ def listen():
         "botocore.patched_stepfunctions_api_call",
         "botocore.patched_bedrock_api_call",
         "redis.command",
+        "redis.execute_pipeline",
         "valkey.command",
         "rq.queue.enqueue_job",
         "rq.traced_queue_fetch_job",
@@ -982,11 +1049,28 @@ def listen():
         "azure.functions.patched_service_bus",
         "azure.functions.patched_timer",
         "azure.servicebus.patched_producer",
+        "psycopg.patched_connect",
     ):
         core.on(f"context.started.{context_name}", _start_span)
 
-    for name in ("django.template.render", "django.process_exception"):
+    for name in (
+        "django.middleware.__call__",
+        "django.middleware.func",
+        "django.middleware.process_exception",
+        "django.middleware.process_request",
+        "django.middleware.process_response",
+        "django.middleware.process_template_response",
+        "django.middleware.process_view",
+        "django.template.render",
+        "molten.trace_func",
+        "redis.execute_pipeline",
+        "redis.command",
+        "psycopg.patched_connect",
+    ):
         core.on(f"context.ended.{name}", _finish_span)
 
+    # Special/extra handling before calling _finish_span
+    core.on("context.ended.django.cache", _on_django_cache)
+
 
 listen()
