@@ -1,9 +1,13 @@
+from ddtrace.appsec._iast._taint_tracking._native.context import finish_request_context
+from ddtrace.appsec._iast._taint_tracking._native.context import start_request_context
 from ddtrace.appsec._iast._taint_tracking._native.initializer import create_context  # noqa: F401
 from ddtrace.appsec._iast._taint_tracking._native.initializer import reset_context  # noqa: F401
 from ddtrace.appsec._iast._taint_tracking._native.initializer import reset_contexts  # noqa: F401
 
 
 __all__ = [
+    "start_request_context",
+    "finish_request_context",
     "create_context",
     "reset_context",
     "reset_contexts",
