@@ -64,10 +64,10 @@ if profiling_config.enabled:
 if config._runtime_metrics_enabled:
     RuntimeWorker.enable()
 
-if config._otel_enabled:
+if config._otel_trace_enabled:
 
     @ModuleWatchdog.after_module_imported("opentelemetry.trace")
-    def _(_):
+    def _ot_traces(_):
         from opentelemetry.trace import set_tracer_provider
 
         from ddtrace.opentelemetry import TracerProvider
@@ -84,6 +84,16 @@ if config._otel_metrics_enabled:
         set_otel_meter_provider()
 
 
+if config._otel_logs_enabled:
+
+    @register_post_preload
+    def _otel_logs():
+        # Post load to ensure that the logger provider is not overridden by module unloading
+        from ddtrace.internal.opentelemetry.logs import set_otel_logs_provider
+
+        set_otel_logs_provider()
+
+
 if config._llmobs_enabled:
     from ddtrace.llmobs import LLMObs
 
