@@ -5,6 +5,7 @@ from celery import signals
 
 from ddtrace import config
 from ddtrace._trace.pin import _DD_PIN_NAME
+from ddtrace._trace.pin import Pin
 from ddtrace.constants import _SPAN_MEASURED_KEY
 from ddtrace.constants import SPAN_KIND
 from ddtrace.contrib import trace_utils
@@ -18,7 +19,6 @@ from ddtrace.ext import SpanKind
 from ddtrace.ext import SpanTypes
 from ddtrace.internal import core
 from ddtrace.internal.logger import get_logger
-from ddtrace.trace import Pin
 
 
 log = get_logger(__name__)
@@ -126,13 +126,13 @@ def _traced_apply_async_function(integration_config, fn_name, resource_fn=None):
             except Exception:
                 # If an internal exception occurs, record the exception in the span,
                 # then raise the Celery error as usual
-                task_span = core.get_item("task_span")
+                task_span = core.find_item("task_span")
                 if task_span:
                     task_span.set_exc_info(*sys.exc_info())
 
                 raise
             finally:
-                task_span = core.get_item("task_span")
+                task_span = core.find_item("task_span")
                 if task_span:
                     log.debug(
                         "The after_task_publish signal was not called, so manually closing span: %r",
