@@ -13,12 +13,14 @@ from wrapt import ObjectProxy
 from wrapt import wrap_function_wrapper as _w
 
 from ddtrace import config
+from ddtrace._trace.pin import Pin
 from ddtrace.contrib import trace_utils
 from ddtrace.contrib.asgi import TraceMiddleware
 from ddtrace.contrib.internal.trace_utils import with_traced_module
 from ddtrace.ext import http
 from ddtrace.internal import core
 from ddtrace.internal._exceptions import BlockingException
+from ddtrace.internal.endpoints import endpoint_collection
 from ddtrace.internal.logger import get_logger
 from ddtrace.internal.schema import schematize_service_name
 from ddtrace.internal.telemetry import get_config as _get_config
@@ -28,7 +30,6 @@ from ddtrace.internal.utils import set_argument_value
 from ddtrace.internal.utils.formats import asbool
 from ddtrace.internal.utils.wrappers import unwrap as _u
 from ddtrace.settings.asm import config as asm_config
-from ddtrace.trace import Pin
 from ddtrace.trace import Span  # noqa:F401
 from ddtrace.vendor.packaging.version import parse as parse_version
 
@@ -62,6 +63,7 @@ def get_version():
 
 
 _STARLETTE_VERSION = parse_version(get_version())
+_STARLETTE_VERSION_LTE_0_33_0 = _STARLETTE_VERSION <= parse_version("0.33.0")
 
 
 def _supported_versions() -> Dict[str, str]:
@@ -77,10 +79,22 @@ def traced_init(wrapped, instance, args, kwargs):
 
 
 def traced_route_init(wrapped, _instance, args, kwargs):
+    route = args[0] if args else None
+    if route is not None:
+        response_body_type = getattr(kwargs.get("response_class", None), "media_type", None)
+        response_body_type = [response_body_type] if isinstance(response_body_type, str) else []
+        response_code = kwargs.get("status_code", None)
+        response_code = [response_code] if isinstance(response_code, int) else []
+        for m in kwargs.get("methods", None) or []:
+            endpoint_collection.add_endpoint(
+                m,
+                route,
+                operation_name="fastapi.request",
+                response_body_type=response_body_type,
+                response_code=response_code,
+            )
     handler = get_argument_value(args, kwargs, 1, "endpoint")
-
     core.dispatch("service_entrypoint.patch", (inspect.unwrap(handler),))
-
     return wrapped(*args, **kwargs)
 
 
@@ -202,7 +216,7 @@ def traced_handler(wrapped, instance, args, kwargs):
         raise BlockingException(blocked)
 
     # https://github.com/encode/starlette/issues/1336
-    if _STARLETTE_VERSION <= parse_version("0.33.0") and len(request_spans) > 1:
+    if _STARLETTE_VERSION_LTE_0_33_0 and len(request_spans) > 1:
         request_spans[-1].set_tag(http.URL, request_spans[0].get_tag(http.URL))
 
     return wrapped(*args, **kwargs)
