@@ -3,8 +3,8 @@ async tracing utils
 
 Note that this module should only be imported in Python 3.5+.
 """
+from ddtrace._trace.pin import Pin
 from ddtrace.internal.logger import get_logger
-from ddtrace.trace import Pin
 
 
 log = get_logger(__name__)
