@@ -17,7 +17,7 @@ from ddtrace._trace.context import Context
 from ddtrace._trace.span import Span  # noqa:F401
 from ddtrace._trace.span import _get_64_highest_order_bits_as_hex
 from ddtrace._trace.span import _get_64_lowest_order_bits_as_int
-from ddtrace._trace.span import _MetaDictType
+from ddtrace._trace.types import _MetaDictType
 from ddtrace.appsec._constants import APPSEC
 from ddtrace.internal import core
 from ddtrace.internal.telemetry import telemetry_writer
@@ -52,8 +52,6 @@ from ..internal.constants import PROPAGATION_STYLE_DATADOG
 from ..internal.constants import W3C_TRACEPARENT_KEY
 from ..internal.constants import W3C_TRACESTATE_KEY
 from ..internal.logger import get_logger
-from ..internal.sampling import SAMPLING_DECISION_TRACE_TAG_KEY
-from ..internal.sampling import SamplingMechanism
 from ..internal.sampling import validate_sampling_decision
 from ..internal.utils.http import w3c_tracestate_add_p
 from ._utils import get_wsgi_header
@@ -222,7 +220,7 @@ class _DatadogMultiHeader:
             meta = {
                 "_dd.propagation_error": "extract_max_size",
             }
-            log.warning("failed to decode x-datadog-tags", exc_info=True)
+            log.warning("failed to decode x-datadog-tags: %r", tags_value, exc_info=True)
         except TagsetDecodeError:
             meta = {
                 "_dd.propagation_error": "decoding_error",
@@ -362,10 +360,6 @@ class _DatadogMultiHeader:
 
         if not meta:
             meta = {}
-
-        if not meta.get(SAMPLING_DECISION_TRACE_TAG_KEY):
-            meta[SAMPLING_DECISION_TRACE_TAG_KEY] = f"-{SamplingMechanism.LOCAL_USER_TRACE_SAMPLING_RULE}"
-
         # Try to parse values into their expected types
         try:
             if sampling_priority is not None:
@@ -677,7 +671,7 @@ class _TraceContext:
 
     The format for ``tracestate`` is key value pairs with each entry limited to 256 characters.
     An example of the ``dd`` list member we would add is::
-    "dd=s:2;o:rum;t.dm:-4;t.usr.id:baz64"
+    "dd=s:2;o:rum;t.dm:-4;t.usr.id:baz64;t.ksr:0.0001"
 
     Implementation details:
       - Datadog Trace and Span IDs are 64-bit unsigned integers.
@@ -931,7 +925,7 @@ class _BaggageHeader:
                 # Record telemetry for baggage item count exceeding limit
                 telemetry_writer.add_count_metric(
                     namespace=TELEMETRY_NAMESPACE.TRACERS,
-                    name="context_header_style.truncated",
+                    name="context_header.truncated",
                     value=1,
                     tags=(("truncation_reason", "baggage_item_count_exceeded"),),
                 )
@@ -947,7 +941,7 @@ class _BaggageHeader:
                     # Record telemetry for baggage header size exceeding limit
                     telemetry_writer.add_count_metric(
                         namespace=TELEMETRY_NAMESPACE.TRACERS,
-                        name="context_header_style.truncated",
+                        name="context_header.truncated",
                         value=1,
                         tags=(("truncation_reason", "baggage_byte_count_exceeded"),),
                     )
