@@ -18,8 +18,9 @@ from ddtrace.ext import SpanTypes
 from ddtrace.internal import core
 from ddtrace.internal.constants import AI_GUARD_ENABLED
 from ddtrace.internal.constants import AI_GUARD_ENDPOINT
+from ddtrace.internal.constants import AI_GUARD_MAX_CONTENT_SIZE
+from ddtrace.internal.constants import AI_GUARD_MAX_HISTORY_LENGTH
 from ddtrace.internal.constants import DD_APPLICATION_KEY
-from ddtrace.internal.endpoints import HttpEndPointsCollection
 from ddtrace.internal.serverless import in_aws_lambda
 from ddtrace.settings._config import config as tracer_config
 from ddtrace.settings._core import DDConfig
@@ -63,9 +64,6 @@ def build_libddwaf_filename() -> str:
     return os.path.join(_DIRNAME, "appsec", "_ddwaf", "libddwaf", ARCHITECTURE, "lib", "libddwaf." + FILE_EXTENSION)
 
 
-endpoint_collection = HttpEndPointsCollection()
-
-
 class ASMConfig(DDConfig):
     _asm_enabled = DDConfig.var(bool, APPSEC_ENV, default=False)
     _asm_enabled_origin = APPSEC.ENABLED_ORIGIN_UNKNOWN
@@ -73,7 +71,7 @@ class ASMConfig(DDConfig):
     # prevent empty string
     if _asm_static_rule_file == "":
         _asm_static_rule_file = None
-    _asm_processed_span_types = {SpanTypes.WEB, SpanTypes.GRPC}
+    _asm_processed_span_types = {SpanTypes.WEB}
     _asm_http_span_types = {SpanTypes.WEB}
     _iast_enabled = tracer_config._from_endpoint.get("iast_enabled", DDConfig.var(bool, IAST.ENV, default=False))
     _iast_propagation_enabled = DDConfig.var(bool, IAST.ENV_PROPAGATION_ENABLED, default=True, private=True)
@@ -156,6 +154,7 @@ class ASMConfig(DDConfig):
     _iast_lazy_taint = DDConfig.var(bool, IAST.LAZY_TAINT, default=False)
     _iast_deduplication_enabled = DDConfig.var(bool, "DD_IAST_DEDUPLICATION_ENABLED", default=True)
     _iast_security_controls = DDConfig.var(str, "DD_IAST_SECURITY_CONTROLS_CONFIGURATION", default="")
+    _iast_use_root_span = DDConfig.var(bool, "_DD_IAST_USE_ROOT_SPAN", default=False)
 
     _iast_is_testing = False
 
@@ -188,6 +187,16 @@ class ASMConfig(DDConfig):
     # Timeout for the request body reading in seconds.
     _fast_api_async_body_timeout = DDConfig.var(float, "DD_FASTAPI_ASYNC_BODY_TIMEOUT_SECONDS", default=0.1)
 
+    # DOWNSTREAM REQUESTS INSTRUMENTATION
+    # sample rate for body analysis
+    _dr_sample_rate: float = DDConfig.var(
+        float, "DD_API_SECURITY_DOWNSTREAM_REQUEST_BODY_ANALYSIS_SAMPLE_RATE", default=0.5
+    )
+    # max number of downstream requests analysis  with bodies per request
+    _dr_body_limit_per_request: int = DDConfig.var(
+        int, "DD_API_SECURITY_MAX_DOWNSTREAM_REQUEST_BODY_ANALYSIS", default=1
+    )
+
     # for tests purposes
     _asm_config_keys = [
         "_asm_enabled",
@@ -205,6 +214,7 @@ class ASMConfig(DDConfig):
         "_iast_telemetry_report_lvl",
         "_iast_security_controls",
         "_iast_is_testing",
+        "_iast_use_root_span",
         "_ep_enabled",
         "_use_metastruct_for_triggers",
         "_use_metastruct_for_iast",
@@ -217,6 +227,8 @@ class ASMConfig(DDConfig):
         "_api_security_enabled",
         "_api_security_sample_delay",
         "_api_security_parse_response_body",
+        "_dr_sample_rate",
+        "_dr_body_limit_per_request",
         "_waf_timeout",
         "_iast_redaction_enabled",
         "_iast_redaction_name_pattern",
@@ -318,7 +330,7 @@ class ASMConfig(DDConfig):
 
     @property
     def is_iast_request_enabled(self) -> bool:
-        env = core.get_item(IAST.REQUEST_CONTEXT_KEY)
+        env = core.find_item(IAST.REQUEST_CONTEXT_KEY)
         if env:
             return env.request_enabled
         return False
@@ -328,9 +340,24 @@ config = ASMConfig()
 
 
 class AIGuardConfig(DDConfig):
-    enabled = DDConfig.var(bool, AI_GUARD_ENABLED, default=True)
-    endpoint = DDConfig.var(str, AI_GUARD_ENDPOINT, default="")
+    _ai_guard_enabled = DDConfig.var(bool, AI_GUARD_ENABLED, default=False)
+    _ai_guard_endpoint = DDConfig.var(str, AI_GUARD_ENDPOINT, default="")
+    _ai_guard_max_history_length = DDConfig.var(int, AI_GUARD_MAX_HISTORY_LENGTH, default=16)
+    _ai_guard_max_content_size = DDConfig.var(int, AI_GUARD_MAX_CONTENT_SIZE, default=512 * 1024)
     _dd_app_key = DDConfig.var(str, DD_APPLICATION_KEY, default="")
 
+    # for tests purposes
+    _ai_guard_config_keys = [
+        "_ai_guard_enabled",
+        "_ai_guard_endpoint",
+        "_ai_guard_max_history_length",
+        "_ai_guard_max_content_size",
+        "_dd_app_key",
+    ]
+
+    def reset(self):
+        """For testing purposes, reset the configuration to its default values given current environment variables."""
+        self.__init__()
+
 
 ai_guard_config = AIGuardConfig()
