@@ -60,11 +60,13 @@ RESULT = "unknown"
 RESULT_REASON = "unknown"
 RESULT_CLASS = "unknown"
 EXECUTABLES_DENY_LIST = set()
+EXECUTABLE_MODULES_DENY_LIST = set()
 REQUIREMENTS_FILE_LOCATIONS = (
     os.path.abspath(os.path.join(SCRIPT_DIR, "../datadog-lib/requirements.csv")),
     os.path.abspath(os.path.join(SCRIPT_DIR, "requirements.csv")),
 )
 EXECUTABLE_DENY_LOCATION = os.path.abspath(os.path.join(SCRIPT_DIR, "denied_executables.txt"))
+EXECUTABLE_MODULES_DENY_LOCATION = os.path.abspath(os.path.join(SCRIPT_DIR, "denied_executable_modules.txt"))
 SITE_PKGS_MARKER = "site-packages-ddtrace-py"
 BOOTSTRAP_MARKER = "bootstrap"
 
@@ -147,6 +149,24 @@ def build_denied_executables():
     return denied_executables
 
 
+def build_denied_executable_modules():
+    denied_modules = set()
+    _log("Checking denied-executable-modules list", level="debug")
+    try:
+        if os.path.exists(EXECUTABLE_MODULES_DENY_LOCATION):
+            with open(EXECUTABLE_MODULES_DENY_LOCATION, "r") as denyfile:
+                _log("Found modules deny-list file", level="debug")
+                for line in denyfile.readlines():
+                    cleaned = line.strip("\n").strip()
+                    # Skip empty lines and comments
+                    if cleaned and not cleaned.startswith("#"):
+                        denied_modules.add(cleaned)
+        _log("Built denied-executable-modules list of %s entries" % (len(denied_modules),), level="debug")
+    except Exception as e:
+        _log("Failed to build denied-executable-modules list: %s" % e, level="debug")
+    return denied_modules
+
+
 def create_count_metric(metric, tags=None):
     if tags is None:
         tags = []
@@ -262,12 +282,30 @@ def get_first_incompatible_sysarg():
     _log("Checking sys.args: len(sys.argv): %s" % (len(sys.argv),), level="debug")
     if len(sys.argv) <= 1:
         return
+
+    # Check the main executable first
     argument = sys.argv[0]
     _log("Is argument %s in deny-list?" % (argument,), level="debug")
     if argument in EXECUTABLES_DENY_LIST or os.path.basename(argument) in EXECUTABLES_DENY_LIST:
         _log("argument %s is in deny-list" % (argument,), level="debug")
         return argument
 
+    # Check for "-m module" patterns, but only for Python interpreters
+    if len(sys.argv) >= 3:
+        executable_basename = os.path.basename(argument)
+        if executable_basename.startswith("python"):
+            try:
+                m_index = sys.argv.index("-m")
+                if m_index + 1 < len(sys.argv):
+                    module_name = sys.argv[m_index + 1]
+                    if module_name in EXECUTABLE_MODULES_DENY_LIST:
+                        _log("Module %s is in deny-list" % (module_name,), level="debug")
+                        return "-m %s" % module_name
+            except ValueError:
+                # "-m" not found in sys.argv, continue normally
+                pass
+    return None
+
 
 def _inject():
     global DDTRACE_VERSION
@@ -276,6 +314,7 @@ def _inject():
     global PYTHON_RUNTIME
     global DDTRACE_REQUIREMENTS
     global EXECUTABLES_DENY_LIST
+    global EXECUTABLE_MODULES_DENY_LIST
     global TELEMETRY_DATA
     global RESULT
     global RESULT_REASON
@@ -287,6 +326,7 @@ def _inject():
     INSTALLED_PACKAGES = build_installed_pkgs()
     DDTRACE_REQUIREMENTS = build_requirements(PYTHON_VERSION)
     EXECUTABLES_DENY_LIST = build_denied_executables()
+    EXECUTABLE_MODULES_DENY_LIST = build_denied_executable_modules()
     dependency_incomp = False
     runtime_incomp = False
     spec = None
