@@ -11,6 +11,10 @@ config._add(
 )
 
 
+def _supported_versions() -> dict[str, str]:
+    return {"fooo": ">=0.0.0"}
+
+
 def get_version():
     # get the package distribution version here
     pass
