@@ -12,6 +12,7 @@ from fastapi.responses import JSONResponse
 from fastapi.responses import StreamingResponse
 from pydantic import BaseModel
 
+from ddtrace._trace.pin import Pin
 import ddtrace.constants
 from ddtrace.trace import tracer
 
@@ -51,16 +52,27 @@ def get_app():
         """
         return await call_next(request)
 
-    @app.get("/")
-    @app.post("/")
-    @app.options("/")
-    async def read_homepage():  # noqa: B008
-        return HTMLResponse("ok ASM", 200)
+    @app.middleware("http")
+    async def rename_service(request: Request, call_next):
+        if request.headers.get("x-rename-service", "false").lower() == "true":
+            service_name = "sub-service"
+            root_span = tracer.current_root_span()
+            if root_span is not None:
+                root_span.service = service_name
+                root_span.set_tag("scope", service_name)
 
-    @app.get("/asm/{param_int:int}/{param_str:str}/")
-    @app.post("/asm/{param_int:int}/{param_str:str}/")
-    @app.get("/asm/{param_int:int}/{param_str:str}")
-    @app.post("/asm/{param_int:int}/{param_str:str}")
+        return await call_next(request)
+
+    @app.get("/", response_class=HTMLResponse, status_code=200)
+    @app.post("/", response_class=HTMLResponse, status_code=200)
+    @app.options("/", response_class=HTMLResponse, status_code=200)
+    async def read_homepage():  # noqa: B008
+        return "ok ASM"
+
+    @app.get("/asm/{param_int:int}/{param_str:str}/", response_class=JSONResponse)
+    @app.post("/asm/{param_int:int}/{param_str:str}/", response_class=JSONResponse)
+    @app.get("/asm/{param_int:int}/{param_str:str}", response_class=JSONResponse)
+    @app.post("/asm/{param_int:int}/{param_str:str}", response_class=JSONResponse)
     async def multi_view(param_int: int, param_str: str, request: Request):  # noqa: B008
         query_params = dict(request.query_params)
         body = {
@@ -117,13 +129,13 @@ def get_app():
     async def new_service(service_name: str, request: Request):  # noqa: B008
         import ddtrace
 
-        ddtrace.trace.Pin._override(app, service=service_name, tracer=ddtrace.tracer)
+        Pin._override(app, service=service_name, tracer=ddtrace.tracer)
         return HTMLResponse(service_name, 200)
 
     async def slow_numbers(minimum, maximum):
         for number in range(minimum, maximum):
             yield "%d" % number
-            await asyncio.sleep(0.25)
+            await asyncio.sleep(0.0625)
 
     @app.get("/stream/")
     async def stream():
@@ -149,11 +161,12 @@ def get_app():
                         res.append(f"File: {f.read()}")
                 except Exception as e:
                     res.append(f"Error: {e}")
-            tracer.current_span()._local_root.set_tag("rasp.request.done", endpoint)
+            tracer.current_span()._service_entry_span.set_tag("rasp.request.done", endpoint)
             return HTMLResponse("<\br>\n".join(res))
         elif endpoint == "ssrf":
             res = ["ssrf endpoint"]
             for param in query_params:
+                urlname = ""
                 if param.startswith("url"):
                     urlname = query_params[param]
                     if not urlname.startswith("http"):
@@ -162,22 +175,22 @@ def get_app():
                     if param.startswith("url_urlopen_request"):
                         import urllib.request
 
-                        request = urllib.request.Request(urlname)
-                        with urllib.request.urlopen(request, timeout=0.15) as f:
+                        request_urllib = urllib.request.Request(urlname, method="GET")
+                        with urllib.request.urlopen(request_urllib, timeout=0.5) as f:
                             res.append(f"Url: {f.read()}")
                     elif param.startswith("url_urlopen_string"):
                         import urllib.request
 
-                        with urllib.request.urlopen(urlname, timeout=0.15) as f:
+                        with urllib.request.urlopen(urlname, timeout=0.5) as f:
                             res.append(f"Url: {f.read()}")
                     elif param.startswith("url_requests"):
                         import requests
 
-                        r = requests.get(urlname, timeout=0.15)
+                        r = requests.get(urlname, timeout=0.5)
                         res.append(f"Url: {r.text}")
                 except Exception as e:
                     res.append(f"Error: {e}")
-            tracer.current_span()._local_root.set_tag("rasp.request.done", endpoint)
+            tracer.current_span()._service_entry_span.set_tag("rasp.request.done", endpoint)
             return HTMLResponse("<\\br>\n".join(res))
         elif endpoint == "sql_injection":
             res = ["sql_injection endpoint"]
@@ -190,7 +203,7 @@ def get_app():
                         res.append(f"Url: {list(cursor)}")
                 except Exception as e:
                     res.append(f"Error: {e}")
-            tracer.current_span()._local_root.set_tag("rasp.request.done", endpoint)
+            tracer.current_span()._service_entry_span.set_tag("rasp.request.done", endpoint)
             return HTMLResponse("<\\br>\n".join(res))
         elif endpoint == "shell_injection":
             res = ["shell_injection endpoint"]
@@ -199,12 +212,12 @@ def get_app():
                     cmd = query_params[param]
                     try:
                         if param.startswith("cmdsys"):
-                            res.append(f'cmd stdout: {os.system(f"ls {cmd}")}')
+                            res.append(f"cmd stdout: {os.system(f'ls {cmd}')}")
                         else:
-                            res.append(f'cmd stdout: {subprocess.run(f"ls {cmd}", shell=True, timeout=1)}')
+                            res.append(f"cmd stdout: {subprocess.run(f'ls {cmd}', shell=True, timeout=1)}")
                     except Exception as e:
                         res.append(f"Error: {e}")
-            tracer.current_span()._local_root.set_tag("rasp.request.done", endpoint)
+            tracer.current_span()._service_entry_span.set_tag("rasp.request.done", endpoint)
             return HTMLResponse("<\\br>\n".join(res))
         elif endpoint == "command_injection":
             res = ["command_injection endpoint"]
@@ -212,20 +225,48 @@ def get_app():
                 if param.startswith("cmda"):
                     cmd = query_params[param]
                     try:
-                        res.append(f'cmd stdout: {subprocess.run([cmd, "-c", "3", "localhost"], timeout=1)}')
+                        res.append(f"cmd stdout: {subprocess.run([cmd, '-c', '3', 'localhost'], timeout=0.25)}")
                     except Exception as e:
                         res.append(f"Error: {e}")
                 elif param.startswith("cmds"):
                     cmd = query_params[param]
                     try:
-                        res.append(f"cmd stdout: {subprocess.run(cmd, timeout=1)}")
+                        res.append(f"cmd stdout: {subprocess.run(cmd, timeout=0.25)}")
                     except Exception as e:
                         res.append(f"Error: {e}")
-            tracer.current_span()._local_root.set_tag("rasp.request.done", endpoint)
+            tracer.current_span()._service_entry_span.set_tag("rasp.request.done", endpoint)
             return HTMLResponse("<\\br>\n".join(res))
-        tracer.current_span()._local_root.set_tag("rasp.request.done", endpoint)
+        tracer.current_span()._service_entry_span.set_tag("rasp.request.done", endpoint)
         return HTMLResponse(f"Unknown endpoint: {endpoint}")
 
+    @app.get("/redirect/{url:str}/", response_class=JSONResponse)
+    async def redirect_get(url: str, request: Request):
+        import urllib.request
+
+        url = "http://" + url
+        try:
+            request_urllib = urllib.request.Request(url, method="GET")
+            with urllib.request.urlopen(request_urllib, timeout=0.5) as f:
+                payload = {"payload": f.read()}
+        except Exception as e:
+            payload = {"error": repr(e)}
+        return payload
+
+    @app.post("/redirect/{url:str}/", response_class=JSONResponse)
+    async def redirect_post(url: str, request: Request):
+        import urllib.request
+
+        url = "http://" + url
+        try:
+            request_urllib = urllib.request.Request(
+                url, method="POST", data=(await request.body()), headers={"Content-Type": "application/json"}
+            )
+            with urllib.request.urlopen(request_urllib, timeout=0.5) as f:
+                payload = {"payload": f.read()}
+        except Exception as e:
+            payload = {"error": repr(e)}
+        return payload
+
     @app.get("/login/")
     async def login_user(request: Request):
         """manual instrumentation login endpoint"""
