@@ -1,9 +1,9 @@
 from ddtrace import config
 from ddtrace._monkey import _patch_all
+from ddtrace._trace.pin import Pin
 from ddtrace.contrib.internal.algoliasearch.patch import algoliasearch_version
 from ddtrace.contrib.internal.algoliasearch.patch import patch
 from ddtrace.contrib.internal.algoliasearch.patch import unpatch
-from ddtrace.trace import Pin
 from ddtrace.vendor.packaging.version import parse as parse_version
 from tests.utils import TracerTestCase
 from tests.utils import assert_is_measured
