@@ -4,12 +4,13 @@ import mock
 from mock import patch as mock_patch
 import pytest
 
+from ddtrace._trace.pin import Pin
 from ddtrace.llmobs import LLMObs
 from ddtrace.llmobs import LLMObs as llmobs_service
-from ddtrace.trace import Pin
 from tests.contrib.botocore.bedrock_utils import _MODELS
 from tests.contrib.botocore.bedrock_utils import _REQUEST_BODIES
 from tests.contrib.botocore.bedrock_utils import BOTO_VERSION
+from tests.contrib.botocore.bedrock_utils import FETCH_CONCEPT_TOOL_DEFINITION
 from tests.contrib.botocore.bedrock_utils import bedrock_converse_args_with_system_and_tool
 from tests.contrib.botocore.bedrock_utils import create_bedrock_converse_request
 from tests.contrib.botocore.bedrock_utils import get_mock_response_data
@@ -268,6 +269,7 @@ class TestLLMObsBedrock:
                             "arguments": {"concept": "distributed tracing"},
                             "name": "fetch_concept",
                             "tool_id": mock.ANY,
+                            "type": "toolUse",
                         }
                     ],
                 }
@@ -282,6 +284,7 @@ class TestLLMObsBedrock:
                 "output_tokens": response["usage"]["outputTokens"],
                 "total_tokens": response["usage"]["totalTokens"],
             },
+            tool_definitions=[FETCH_CONCEPT_TOOL_DEFINITION],
             tags={"service": "aws.bedrock-runtime", "ml_app": "<ml-app-name>"},
         )
 
@@ -346,6 +349,7 @@ class TestLLMObsBedrock:
                             "arguments": {"concept": "distributed tracing"},
                             "name": "fetch_concept",
                             "tool_id": mock.ANY,
+                            "type": "toolUse",
                         }
                     ],
                 }
@@ -359,6 +363,7 @@ class TestLLMObsBedrock:
                 "output_tokens": 64,
                 "total_tokens": 323,
             },
+            tool_definitions=[FETCH_CONCEPT_TOOL_DEFINITION],
             tags={"service": "aws.bedrock-runtime", "ml_app": "<ml-app-name>"},
         )
 
@@ -398,6 +403,7 @@ class TestLLMObsBedrock:
                             "arguments": {"concept": "distributed tracing"},
                             "name": "fetch_concept",
                             "tool_id": mock.ANY,
+                            "type": "toolUse",
                         }
                     ],
                 }
@@ -411,6 +417,7 @@ class TestLLMObsBedrock:
                 "output_tokens": 64,
                 "total_tokens": 323,
             },
+            tool_definitions=[FETCH_CONCEPT_TOOL_DEFINITION],
             tags={"service": "aws.bedrock-runtime", "ml_app": "<ml-app-name>"},
         )
 
@@ -581,7 +588,9 @@ class TestLLMObsBedrock:
             )
 
         assert len(llmobs_events) == 1
-        assert llmobs_events[0]["meta"]["input"]["messages"] == [{"content": "bar", "role": "tool", "tool_id": "foo"}]
+        assert llmobs_events[0]["meta"]["input"]["messages"] == [
+            {"tool_results": [{"result": "bar", "tool_id": "foo", "type": "toolResult"}], "role": "user"}
+        ]
 
     @pytest.mark.skipif(BOTO_VERSION < (1, 34, 131), reason="Converse API not available until botocore 1.34.131")
     def test_llmobs_converse_tool_result_json(self, bedrock_client, request_vcr, mock_tracer, llmobs_events):
@@ -601,7 +610,7 @@ class TestLLMObsBedrock:
 
         assert len(llmobs_events) == 1
         assert llmobs_events[0]["meta"]["input"]["messages"] == [
-            {"content": '{"result": "bar"}', "role": "tool", "tool_id": "foo"}
+            {"tool_results": [{"result": '{"result": "bar"}', "tool_id": "foo", "type": "toolResult"}], "role": "user"}
         ]
 
     @pytest.mark.skipif(BOTO_VERSION < (1, 34, 131), reason="Converse API not available until botocore 1.34.131")
@@ -638,7 +647,12 @@ class TestLLMObsBedrock:
 
         assert len(llmobs_events) == 1
         assert llmobs_events[0]["meta"]["input"]["messages"] == [
-            {"content": "[Unsupported content type(s): image]", "role": "tool", "tool_id": "foo"}
+            {
+                "tool_results": [
+                    {"result": "[Unsupported content type(s): image]", "tool_id": "foo", "type": "toolResult"}
+                ],
+                "role": "user",
+            }
         ]
 
 
