@@ -1,6 +1,6 @@
 import unittest
 
-from ddtrace.trace import Pin
+from ddtrace._trace.pin import Pin
 from tests.contrib.patch import emit_integration_and_version_to_test_agent
 
 
