@@ -1,13 +1,13 @@
 import mock
 import pytest
 
+from ddtrace._trace.pin import Pin
 from ddtrace.contrib.dbapi import FetchTracedCursor
 from ddtrace.contrib.dbapi import TracedConnection
 from ddtrace.contrib.dbapi import TracedCursor
 from ddtrace.propagation._database_monitoring import _DBM_Propagator
 from ddtrace.settings._config import Config
 from ddtrace.settings.integration import IntegrationConfig
-from ddtrace.trace import Pin
 from ddtrace.trace import Span  # noqa:F401
 from tests.utils import TracerTestCase
 from tests.utils import assert_is_measured
