@@ -2,9 +2,9 @@ import flask
 from flask.testing import FlaskClient
 import wrapt
 
+from ddtrace._trace.pin import Pin
 from ddtrace.contrib.internal.flask.patch import patch
 from ddtrace.contrib.internal.flask.patch import unpatch
-from ddtrace.trace import Pin
 from tests.utils import TracerTestCase
 
 
