@@ -1,6 +1,4 @@
 from operator import itemgetter
-import os
-import sys
 
 import langchain
 import langchain.prompts  # noqa: F401
@@ -8,7 +6,6 @@ import mock
 import pytest
 
 from ddtrace.internal.utils.version import parse_version
-from tests.contrib.langchain.utils import get_request_vcr
 
 
 LANGCHAIN_VERSION = parse_version(langchain.__version__)
@@ -31,20 +28,15 @@ IGNORE_FIELDS = [
 ]
 
 
-@pytest.fixture(scope="session")
-def request_vcr():
-    yield get_request_vcr()
-
-
 @pytest.mark.snapshot(ignores=IGNORE_FIELDS)
-def test_openai_llm_sync(langchain_openai, openai_completion):
-    llm = langchain_openai.OpenAI()
+def test_openai_llm_sync(langchain_openai, openai_url):
+    llm = langchain_openai.OpenAI(base_url=openai_url)
     llm.invoke("Can you explain what Descartes meant by 'I think, therefore I am'?")
 
 
 @pytest.mark.snapshot(ignores=IGNORE_FIELDS)
-def test_openai_llm_sync_multiple_prompts(langchain_openai, openai_completion_multiple):
-    llm = langchain_openai.OpenAI()
+def test_openai_llm_sync_multiple_prompts(langchain_openai, openai_url):
+    llm = langchain_openai.OpenAI(base_url=openai_url)
     llm.generate(
         prompts=[
             "What is the best way to teach a baby multiple languages?",
@@ -55,73 +47,39 @@ def test_openai_llm_sync_multiple_prompts(langchain_openai, openai_completion_mu
 
 @pytest.mark.asyncio
 @pytest.mark.snapshot(ignores=IGNORE_FIELDS)
-async def test_openai_llm_async(langchain_openai, openai_completion):
-    llm = langchain_openai.OpenAI()
+async def test_openai_llm_async(langchain_openai, openai_url):
+    llm = langchain_openai.OpenAI(base_url=openai_url)
     await llm.agenerate(["Which team won the 2019 NBA finals?"])
 
 
 @pytest.mark.snapshot(ignores=IGNORE_FIELDS)
-def test_openai_llm_error(langchain, langchain_openai, openai_completion_error):
+def test_openai_llm_error(langchain, langchain_openai, openai_url):
     import openai  # Imported here because the os env OPENAI_API_KEY needs to be set via langchain fixture before import
 
-    llm = langchain_openai.OpenAI()
+    llm = langchain_openai.OpenAI(base_url=openai_url)
 
     with pytest.raises(openai.BadRequestError):
         llm.generate([12345, 123456])
 
 
-@pytest.mark.skipif(not ((0, 2) <= LANGCHAIN_VERSION < (0, 3)), reason="Compatible with langchain==0.2 only")
-@pytest.mark.snapshot
-def test_cohere_llm_sync_0_2(langchain_cohere, request_vcr):
-    llm = langchain_cohere.llms.Cohere(cohere_api_key=os.getenv("COHERE_API_KEY", "<not-a-real-key>"))
-    with request_vcr.use_cassette("cohere_completion_sync_0_2.yaml"):
-        llm.invoke("What is the secret Krabby Patty recipe?")
-
-
-@pytest.mark.skip(reason="https://github.com/langchain-ai/langchain-cohere/issues/44")
-@pytest.mark.skipif(LANGCHAIN_VERSION < (0, 3), reason="Compatible with langchain>=0.3 only")
-@pytest.mark.snapshot
-def test_cohere_llm_sync_latest(langchain_cohere, request_vcr):
-    with request_vcr.use_cassette("cohere_completion_sync_latest.yaml"):
-        llm = langchain_cohere.llms.Cohere(
-            cohere_api_key=os.getenv("COHERE_API_KEY", "<not-a-real-key>"),
-            max_tokens=256,
-            temperature=0.75,
-        )
-        llm.invoke("What is the secret Krabby Patty recipe?")
-
-
-@pytest.mark.skipif(
-    LANGCHAIN_VERSION < (0, 2) or sys.version_info < (3, 10),
-    reason="Requires separate cassette for langchain v0.1, Python 3.9",
-)
-@pytest.mark.snapshot
-def test_ai21_llm_sync(langchain_community, request_vcr):
-    if langchain_community is None:
-        pytest.skip("langchain-community not installed which is required for this test.")
-    llm = langchain_community.llms.AI21(ai21_api_key=os.getenv("AI21_API_KEY", "<not-a-real-key>"))
-    with request_vcr.use_cassette("ai21_completion_sync.yaml"):
-        llm.invoke("Why does everyone in Bikini Bottom hate Plankton?")
-
-
 @pytest.mark.snapshot(ignores=IGNORE_FIELDS)
-def test_openai_chat_model_sync_call_langchain_openai(langchain_openai, openai_chat_completion):
-    chat = langchain_openai.ChatOpenAI(temperature=0, max_tokens=256, n=1)
+def test_openai_chat_model_sync_call_langchain_openai(langchain_openai, openai_url):
+    chat = langchain_openai.ChatOpenAI(temperature=0, max_tokens=256, n=1, base_url=openai_url)
     chat.invoke(input=[langchain.schema.HumanMessage(content="When do you use 'whom' instead of 'who'?")])
 
 
 @pytest.mark.asyncio
 @pytest.mark.snapshot(ignores=IGNORE_FIELDS)
-async def test_openai_chat_model_sync_call_langchain_openai_async(langchain_openai, openai_chat_completion):
-    chat = langchain_openai.ChatOpenAI(temperature=0, max_tokens=256, n=1)
+async def test_openai_chat_model_sync_call_langchain_openai_async(langchain_openai, openai_url):
+    chat = langchain_openai.ChatOpenAI(temperature=0, max_tokens=256, n=1, base_url=openai_url)
     await chat.ainvoke(input=[langchain.schema.HumanMessage(content="When do you use 'whom' instead of 'who'?")])
 
 
 # TODO: come back and clean this one up... seems like we tag 4 responses
 @pytest.mark.skipif(LANGCHAIN_VERSION < (0, 3), reason="Requires at least LangChain 0.3")
 @pytest.mark.snapshot(ignores=IGNORE_FIELDS)
-def test_openai_chat_model_sync_generate(langchain_openai, openai_chat_completion_multiple):
-    chat = langchain_openai.ChatOpenAI(temperature=0, max_tokens=256)
+def test_openai_chat_model_sync_generate(langchain_openai, openai_url):
+    chat = langchain_openai.ChatOpenAI(temperature=0, max_tokens=256, base_url=openai_url)
     chat.generate(
         [
             [
@@ -143,7 +101,7 @@ def test_openai_chat_model_sync_generate(langchain_openai, openai_chat_completio
         "latest": LANGCHAIN_VERSION >= (0, 3),
     },
 )
-def test_openai_chat_model_vision_generate(langchain_openai, request_vcr):
+def test_openai_chat_model_vision_generate(langchain_openai, openai_url):
     """
     Test that input messages with nested contents are still tagged without error
     Regression test for https://github.com/DataDog/dd-trace-py/issues/8149.
@@ -152,23 +110,22 @@ def test_openai_chat_model_vision_generate(langchain_openai, request_vcr):
         "https://upload.wikimedia.org/wikipedia/commons/thumb/d/dd/Gfp-wisconsin-madison-the-nature-boardwalk"
         ".jpg/2560px-Gfp-wisconsin-madison-the-nature-boardwalk.jpg"
     )
-    chat = langchain_openai.ChatOpenAI(model="gpt-4o", temperature=0, max_tokens=256)
-    with request_vcr.use_cassette("openai_chat_completion_image_input_sync_generate.yaml"):
-        chat.generate(
+    chat = langchain_openai.ChatOpenAI(model="gpt-4o", temperature=0, max_tokens=256, base_url=openai_url)
+    chat.generate(
+        [
             [
-                [
-                    langchain.schema.HumanMessage(
-                        content=[
-                            {"type": "text", "text": "What’s in this image?"},
-                            {
-                                "type": "image_url",
-                                "image_url": {"url": image_url},
-                            },
-                        ],
-                    ),
-                ],
-            ]
-        )
+                langchain.schema.HumanMessage(
+                    content=[
+                        {"type": "text", "text": "What’s in this image?"},
+                        {
+                            "type": "image_url",
+                            "image_url": {"url": image_url},
+                        },
+                    ],
+                ),
+            ],
+        ]
+    )
 
 
 @pytest.mark.skipif(
@@ -183,29 +140,26 @@ def test_openai_chat_model_vision_generate(langchain_openai, request_vcr):
         "meta.langchain.response.completions.1.0.content",
     ]
 )
-async def test_openai_chat_model_async_generate(langchain_openai, request_vcr):
-    chat = langchain_openai.ChatOpenAI(temperature=0, max_tokens=256)
-    with request_vcr.use_cassette("openai_chat_completion_async_generate.yaml"):
-        await chat.agenerate(
+async def test_openai_chat_model_async_generate(langchain_openai, openai_url):
+    chat = langchain_openai.ChatOpenAI(temperature=0, max_tokens=256, base_url=openai_url)
+    await chat.agenerate(
+        [
             [
-                [
-                    langchain.schema.SystemMessage(content="Respond like a frat boy."),
-                    langchain.schema.HumanMessage(
-                        content="Where's the nearest equinox gym from Hudson Yards manhattan?"
-                    ),
-                ],
-                [
-                    langchain.schema.SystemMessage(content="Respond with a pirate accent."),
-                    langchain.schema.HumanMessage(content="How does one get to Bikini Bottom from New York?"),
-                ],
-            ]
-        )
+                langchain.schema.SystemMessage(content="Respond like a frat boy."),
+                langchain.schema.HumanMessage(content="Where's the nearest equinox gym from Hudson Yards manhattan?"),
+            ],
+            [
+                langchain.schema.SystemMessage(content="Respond with a pirate accent."),
+                langchain.schema.HumanMessage(content="How does one get to Bikini Bottom from New York?"),
+            ],
+        ]
+    )
 
 
 @pytest.mark.snapshot
-def test_openai_embedding_query(langchain_openai, openai_embedding):
+def test_openai_embedding_query(langchain_openai, openai_url):
     with mock.patch("langchain_openai.OpenAIEmbeddings._get_len_safe_embeddings", return_value=[0.0] * 1536):
-        embeddings = langchain_openai.OpenAIEmbeddings()
+        embeddings = langchain_openai.OpenAIEmbeddings(base_url=openai_url)
         embeddings.embed_query("this is a test query.")
 
 
@@ -225,46 +179,32 @@ def test_fake_embedding_document(langchain_community):
     embeddings.embed_documents(texts=["foo", "bar"])
 
 
+@pytest.mark.skip("llmobs needs to support in-memory vectorstores")
 @pytest.mark.snapshot
-def test_pinecone_vectorstore_similarity_search(langchain_openai, request_vcr):
-    """
-    Test that calling a similarity search on a Pinecone vectorstore with langchain will
-    result in a 2-span trace with a vectorstore span and underlying OpenAI embedding interface span.
-    """
-    import langchain_pinecone
-    import pinecone
-
-    with mock.patch("langchain_openai.OpenAIEmbeddings._get_len_safe_embeddings", return_value=[0.0] * 1536):
-        with request_vcr.use_cassette("openai_pinecone_similarity_search.yaml"):
-            pc = pinecone.Pinecone(
-                api_key=os.getenv("PINECONE_API_KEY", "<not-a-real-key>"),
-                environment=os.getenv("PINECONE_ENV", "<not-a-real-env>"),
-            )
-            embed = langchain_openai.OpenAIEmbeddings(model="text-embedding-ada-002")
-            index = pc.Index("langchain-retrieval")
-            vectorstore = langchain_pinecone.PineconeVectorStore(index, embed, "text")
-            vectorstore.similarity_search("Who was Alan Turing?", 1)
+def test_vectorstore_similarity_search(langchain_in_memory_vectorstore):
+    vectorstore = langchain_in_memory_vectorstore
+    vectorstore.similarity_search("What is the capital of France?", k=1)
 
 
 @pytest.mark.snapshot(ignores=IGNORE_FIELDS)
-def test_lcel_chain_simple(langchain_core, langchain_openai, openai_completion):
+def test_lcel_chain_simple(langchain_core, langchain_openai, openai_url):
     prompt = langchain_core.prompts.ChatPromptTemplate.from_messages(
         [("system", "You are world class technical documentation writer."), ("user", "{input}")]
     )
-    llm = langchain_openai.OpenAI()
+    llm = langchain_openai.OpenAI(base_url=openai_url)
 
     chain = prompt | llm
     chain.invoke({"input": "how can langsmith help with testing?"})
 
 
 @pytest.mark.snapshot(ignores=IGNORE_FIELDS)
-def test_lcel_chain_complicated(langchain_core, langchain_openai, openai_chat_completion):
+def test_lcel_chain_complicated(langchain_core, langchain_openai, openai_url):
     prompt = langchain_core.prompts.ChatPromptTemplate.from_template(
         "Tell me a short joke about {topic} in the style of {style}"
     )
 
-    chat_openai = langchain_openai.ChatOpenAI(temperature=0.7, n=1)
-    openai = langchain_openai.OpenAI()
+    chat_openai = langchain_openai.ChatOpenAI(temperature=0.7, n=1, base_url=openai_url)
+    openai = langchain_openai.OpenAI(base_url=openai_url)
 
     model = chat_openai.configurable_alternatives(
         langchain_core.runnables.ConfigurableField(id="model"),
@@ -287,25 +227,25 @@ def test_lcel_chain_complicated(langchain_core, langchain_openai, openai_chat_co
 
 @pytest.mark.asyncio
 @pytest.mark.snapshot(ignores=IGNORE_FIELDS)
-async def test_lcel_chain_simple_async(langchain_core, langchain_openai, openai_completion):
+async def test_lcel_chain_simple_async(langchain_core, langchain_openai, openai_url):
     prompt = langchain_core.prompts.ChatPromptTemplate.from_messages(
         [("system", "You are world class technical documentation writer."), ("user", "{input}")]
     )
-    llm = langchain_openai.OpenAI()
+    llm = langchain_openai.OpenAI(base_url=openai_url)
 
     chain = prompt | llm
     await chain.ainvoke({"input": "how can langsmith help with testing?"})
 
 
 @pytest.mark.snapshot(ignores=IGNORE_FIELDS)
-def test_lcel_chain_batch(langchain_core, langchain_openai, openai_chat_completion):
+def test_lcel_chain_batch(langchain_core, langchain_openai, openai_url):
     """
     Test that invoking a chain with a batch of inputs will result in a 4-span trace,
     with a root RunnableSequence span, then 3 LangChain ChatOpenAI spans underneath
     """
     prompt = langchain_core.prompts.ChatPromptTemplate.from_template("Tell me a short joke about {topic}")
     output_parser = langchain_core.output_parsers.StrOutputParser()
-    model = langchain_openai.ChatOpenAI(temperature=0.7, n=1)
+    model = langchain_openai.ChatOpenAI(temperature=0.7, n=1, base_url=openai_url)
     chain = {"topic": langchain_core.runnables.RunnablePassthrough()} | prompt | model | output_parser
 
     chain.batch(inputs=["chickens"])
@@ -340,14 +280,14 @@ def test_lcel_chain_nested(langchain_core, langchain_openai, openai_url):
 
 @pytest.mark.asyncio
 @pytest.mark.snapshot(ignores=IGNORE_FIELDS)
-async def test_lcel_chain_batch_async(langchain_core, langchain_openai, openai_chat_completion):
+async def test_lcel_chain_batch_async(langchain_core, langchain_openai, openai_url):
     """
     Test that invoking a chain with a batch of inputs will result in a 4-span trace,
     with a root RunnableSequence span, then 3 LangChain ChatOpenAI spans underneath
     """
     prompt = langchain_core.prompts.ChatPromptTemplate.from_template("Tell me a short joke about {topic}")
     output_parser = langchain_core.output_parsers.StrOutputParser()
-    model = langchain_openai.ChatOpenAI(temperature=0.7, n=1)
+    model = langchain_openai.ChatOpenAI(temperature=0.7, n=1, base_url=openai_url)
     chain = {"topic": langchain_core.runnables.RunnablePassthrough()} | prompt | model | output_parser
 
     await chain.abatch(inputs=["chickens"])
@@ -366,7 +306,7 @@ def test_lcel_chain_non_dict_input(langchain_core):
 
 
 @pytest.mark.snapshot(ignores=IGNORE_FIELDS)
-def test_lcel_with_tools_openai(langchain_core, langchain_openai, openai_chat_completion_tools):
+def test_lcel_with_tools_openai(langchain_core, langchain_openai, openai_url):
     import langchain_core.tools
 
     @langchain_core.tools.tool
@@ -379,13 +319,13 @@ def test_lcel_with_tools_openai(langchain_core, langchain_openai, openai_chat_co
         """
         return a + b
 
-    llm = langchain_openai.ChatOpenAI(model="gpt-3.5-turbo-0125", temperature=0.7, n=1)
+    llm = langchain_openai.ChatOpenAI(model="gpt-3.5-turbo-0125", temperature=0.7, n=1, base_url=openai_url)
     llm_with_tools = llm.bind_tools([add])
     llm_with_tools.invoke("What is the sum of 1 and 2?")
 
 
 @pytest.mark.snapshot(ignores=IGNORE_FIELDS)
-def test_lcel_with_tools_anthropic(langchain_core, langchain_anthropic, request_vcr):
+def test_lcel_with_tools_anthropic(langchain_core, langchain_anthropic, anthropic_url):
     import langchain_core.tools
 
     @langchain_core.tools.tool
@@ -398,25 +338,33 @@ def test_lcel_with_tools_anthropic(langchain_core, langchain_anthropic, request_
         """
         return a + b
 
-    llm = langchain_anthropic.ChatAnthropic(temperature=1, model_name="claude-3-opus-20240229")
+    kwargs = dict(
+        temperature=1,
+        model_name="claude-3-opus-20240229",
+    )
+
+    if "anthropic_api_url" in langchain_anthropic.ChatAnthropic.__fields__:
+        kwargs["anthropic_api_url"] = anthropic_url
+    else:
+        kwargs["base_url"] = anthropic_url
+
+    llm = langchain_anthropic.ChatAnthropic(**kwargs)
     llm_with_tools = llm.bind_tools([add])
-    with request_vcr.use_cassette("lcel_with_tools_anthropic.yaml"):
-        llm_with_tools.invoke("What is the sum of 1 and 2?")
+    llm_with_tools.invoke("What is the sum of 1 and 2?")
 
 
 @pytest.mark.snapshot
-def test_faiss_vectorstore_retrieval(langchain_community, langchain_openai, request_vcr):
+def test_faiss_vectorstore_retrieval(langchain_community, langchain_openai, openai_url):
     if langchain_community is None:
         pytest.skip("langchain-community not installed which is required for this test.")
     pytest.importorskip("faiss", reason="faiss required for this test.")
     with mock.patch("langchain_openai.OpenAIEmbeddings._get_len_safe_embeddings", return_value=[[0.0] * 1536]):
         faiss = langchain_community.vectorstores.faiss.FAISS.from_texts(
             ["this is a test query."],
-            embedding=langchain_openai.OpenAIEmbeddings(),
+            embedding=langchain_openai.OpenAIEmbeddings(base_url=openai_url),
         )
         retriever = faiss.as_retriever()
-        with request_vcr.use_cassette("openai_retrieval_embedding.yaml"):
-            retriever.invoke("What was the message of the last test query?")
+        retriever.invoke("What was the message of the last test query?")
 
 
 @pytest.mark.snapshot(
@@ -425,19 +373,11 @@ def test_faiss_vectorstore_retrieval(langchain_community, langchain_openai, requ
         + IGNORE_FIELDS
     )
 )
-def test_streamed_chain(langchain_core, langchain_openai, streamed_response_responder):
-    client = streamed_response_responder(
-        module="openai",
-        client_class_key="OpenAI",
-        http_client_key="http_client",
-        endpoint_path=["chat", "completions"],
-        file="lcel_openai_chat_streamed_response.txt",
-    )
-
+def test_streamed_chain(langchain_core, langchain_openai, openai_url):
     prompt = langchain_core.prompts.ChatPromptTemplate.from_messages(
         [("system", "You are world class technical documentation writer."), ("user", "{input}")]
     )
-    llm = langchain_openai.ChatOpenAI(client=client)
+    llm = langchain_openai.ChatOpenAI(base_url=openai_url)
     parser = langchain_core.output_parsers.StrOutputParser()
 
     chain = prompt | llm | parser
@@ -451,31 +391,16 @@ def test_streamed_chain(langchain_core, langchain_openai, streamed_response_resp
         + IGNORE_FIELDS
     )
 )
-def test_streamed_chat(langchain_openai, streamed_response_responder):
-    client = streamed_response_responder(
-        module="openai",
-        client_class_key="OpenAI",
-        http_client_key="http_client",
-        endpoint_path=["chat", "completions"],
-        file="lcel_openai_chat_streamed_response.txt",
-    )
-    model = langchain_openai.ChatOpenAI(client=client)
+def test_streamed_chat(langchain_openai, openai_url):
+    model = langchain_openai.ChatOpenAI(base_url=openai_url)
 
     for _ in model.stream(input="how can langsmith help with testing?"):
         pass
 
 
 @pytest.mark.snapshot(ignores=IGNORE_FIELDS)
-def test_streamed_llm(langchain_openai, streamed_response_responder):
-    client = streamed_response_responder(
-        module="openai",
-        client_class_key="OpenAI",
-        http_client_key="http_client",
-        endpoint_path=["completions"],
-        file="lcel_openai_llm_streamed_response.txt",
-    )
-
-    llm = langchain_openai.OpenAI(client=client)
+def test_streamed_llm(langchain_openai, openai_url):
+    llm = langchain_openai.OpenAI(base_url=openai_url)
 
     for _ in llm.stream(input="How do I write technical documentation?"):
         pass
@@ -488,19 +413,11 @@ def test_streamed_llm(langchain_openai, streamed_response_responder):
     ),
     token="tests.contrib.langchain.test_langchain.test_streamed_chain",
 )
-async def test_astreamed_chain(langchain_core, langchain_openai, async_streamed_response_responder):
-    client = async_streamed_response_responder(
-        module="openai",
-        client_class_key="AsyncOpenAI",
-        http_client_key="http_client",
-        endpoint_path=["chat", "completions"],
-        file="lcel_openai_chat_streamed_response.txt",
-    )
-
+async def test_astreamed_chain(langchain_core, langchain_openai, openai_url):
     prompt = langchain_core.prompts.ChatPromptTemplate.from_messages(
         [("system", "You are world class technical documentation writer."), ("user", "{input}")]
     )
-    llm = langchain_openai.ChatOpenAI(async_client=client)
+    llm = langchain_openai.ChatOpenAI(base_url=openai_url)
     parser = langchain_core.output_parsers.StrOutputParser()
 
     chain = prompt | llm | parser
@@ -515,16 +432,8 @@ async def test_astreamed_chain(langchain_core, langchain_openai, async_streamed_
     ),
     token="tests.contrib.langchain.test_langchain.test_streamed_chat",
 )
-async def test_astreamed_chat(langchain_openai, async_streamed_response_responder):
-    client = async_streamed_response_responder(
-        module="openai",
-        client_class_key="AsyncOpenAI",
-        http_client_key="http_client",
-        endpoint_path=["chat", "completions"],
-        file="lcel_openai_chat_streamed_response.txt",
-    )
-
-    model = langchain_openai.ChatOpenAI(async_client=client)
+async def test_astreamed_chat(langchain_openai, openai_url):
+    model = langchain_openai.ChatOpenAI(base_url=openai_url)
 
     async for _ in model.astream(input="how can langsmith help with testing?"):
         pass
@@ -534,16 +443,8 @@ async def test_astreamed_chat(langchain_openai, async_streamed_response_responde
     ignores=IGNORE_FIELDS,
     token="tests.contrib.langchain.test_langchain.test_streamed_llm",
 )
-async def test_astreamed_llm(langchain_openai, async_streamed_response_responder):
-    client = async_streamed_response_responder(
-        module="openai",
-        client_class_key="AsyncOpenAI",
-        http_client_key="http_client",
-        endpoint_path=["completions"],
-        file="lcel_openai_llm_streamed_response.txt",
-    )
-
-    llm = langchain_openai.OpenAI(async_client=client)
+async def test_astreamed_llm(langchain_openai, openai_url):
+    llm = langchain_openai.OpenAI(base_url=openai_url)
 
     async for _ in llm.astream(input="How do I write technical documentation?"):
         pass
@@ -559,16 +460,8 @@ async def test_astreamed_llm(langchain_openai, async_streamed_response_responder
         ]
     )
 )
-def test_streamed_json_output_parser(langchain, langchain_core, langchain_openai, streamed_response_responder):
-    client = streamed_response_responder(
-        module="openai",
-        client_class_key="OpenAI",
-        http_client_key="http_client",
-        endpoint_path=["chat", "completions"],
-        file="lcel_openai_chat_streamed_response_json_output_parser.txt",
-    )
-
-    model = langchain_openai.ChatOpenAI(model="gpt-4o", max_tokens=50, client=client, n=1, temperature=0.7)
+def test_streamed_json_output_parser(langchain, langchain_core, langchain_openai, openai_url):
+    model = langchain_openai.ChatOpenAI(model="gpt-4o", max_tokens=50, base_url=openai_url, n=1, temperature=0.7)
     parser = langchain_core.output_parsers.JsonOutputParser()
 
     chain = model | parser
@@ -589,16 +482,8 @@ def test_streamed_json_output_parser(langchain, langchain_core, langchain_openai
 
 # until we fully support `astream_events`, we do not need a snapshot here
 # this is just a regression test to make sure we don't throw
-async def test_astreamed_events_does_not_throw(langchain_openai, langchain_core, async_streamed_response_responder):
-    client = async_streamed_response_responder(
-        module="openai",
-        client_class_key="AsyncOpenAI",
-        http_client_key="http_client",
-        endpoint_path=["chat", "completions"],
-        file="lcel_openai_chat_streamed_response.txt",
-    )
-
-    model = langchain_openai.ChatOpenAI(async_client=client)
+async def test_astreamed_events_does_not_throw(langchain_openai, langchain_core, openai_url):
+    model = langchain_openai.ChatOpenAI(base_url=openai_url)
     parser = langchain_core.output_parsers.StrOutputParser()
 
     chain = model | parser
