@@ -3,11 +3,11 @@ import time
 import mongoengine
 import pymongo
 
+from ddtrace._trace.pin import Pin
 from ddtrace.contrib.internal.mongoengine.patch import patch
 from ddtrace.contrib.internal.mongoengine.patch import unpatch
 from ddtrace.ext import mongo as mongox
 from ddtrace.internal.schema import DEFAULT_SPAN_SERVICE_NAME
-from ddtrace.trace import Pin
 from tests.opentracer.utils import init_tracer
 from tests.utils import DummyTracer
 from tests.utils import TracerTestCase
