@@ -2,11 +2,10 @@
 import pymemcache
 import pytest
 
+# project
+from ddtrace._trace.pin import Pin
 from ddtrace.contrib.internal.pymemcache.patch import patch
 from ddtrace.contrib.internal.pymemcache.patch import unpatch
-
-# project
-from ddtrace.trace import Pin
 from tests.utils import override_config
 
 from .test_client_mixin import TEST_HOST
