@@ -10,7 +10,6 @@ from unittest import mock
 
 import pytest
 
-from ddtrace.contrib.internal.pytest._utils import _USE_PLUGIN_V2
 from ddtrace.contrib.internal.pytest._utils import _pytest_version_supports_efd
 from ddtrace.internal.ci_visibility._api_client import TestManagementSettings
 from ddtrace.internal.ci_visibility._api_client import TestProperties
@@ -21,6 +20,8 @@ from tests.contrib.pytest.test_pytest import _get_spans_from_list
 from tests.contrib.pytest.utils import assert_stats
 
 
+_USE_PLUGIN_V2 = True
+
 pytestmark = pytest.mark.skipif(
     not (_USE_PLUGIN_V2 and _pytest_version_supports_efd()),
     reason="Quarantine requires v2 of the plugin and pytest >=7.0",
