@@ -7,9 +7,9 @@ import redis
 import redis.asyncio
 from wrapt import ObjectProxy
 
+from ddtrace._trace.pin import Pin
 from ddtrace.contrib.internal.redis.patch import patch
 from ddtrace.contrib.internal.redis.patch import unpatch
-from ddtrace.trace import Pin
 from ddtrace.trace import tracer
 from tests.utils import override_config
 
