@@ -1,10 +1,10 @@
 import sqlalchemy
 from sqlalchemy import text
 
+from ddtrace._trace.pin import Pin
 from ddtrace.contrib.internal.sqlalchemy.patch import get_version
 from ddtrace.contrib.internal.sqlalchemy.patch import patch
 from ddtrace.contrib.internal.sqlalchemy.patch import unpatch
-from ddtrace.trace import Pin
 from tests.contrib.patch import emit_integration_and_version_to_test_agent
 from tests.utils import TracerTestCase
 from tests.utils import assert_is_measured
