@@ -9,7 +9,7 @@ from time import sleep
 import mock
 import pytest
 
-from ddtrace import config
+from ddtrace._trace.product import _convert_rc_trace_sampling_rules
 from ddtrace._trace.sampler import DatadogSampler
 from ddtrace._trace.sampling_rule import SamplingRule
 from ddtrace.internal.remoteconfig import ConfigMetadata
@@ -512,7 +512,7 @@ def test_rc_default_products_registered():
             [
                 SamplingRule(
                     sample_rate=1.0,
-                    service=SamplingRule.NO_RULE,
+                    service=None,
                     name="web.request",
                     resource="*",
                     tags={"care_about": "yes", "region": "us-*"},
@@ -539,7 +539,7 @@ def test_rc_default_products_registered():
                     service="my-service",
                     name="web.request",
                     resource="*",
-                    tags=SamplingRule.NO_RULE,
+                    tags=None,
                     provenance="customer",
                 )
             ],
@@ -562,7 +562,7 @@ def test_rc_default_products_registered():
                     sample_rate=1.0,
                     service="my-service",
                     name="web.request",
-                    resource=SamplingRule.NO_RULE,
+                    resource=None,
                     tags={"care_about": "yes", "region": "us-*"},
                     provenance="customer",
                 )
@@ -585,7 +585,7 @@ def test_rc_default_products_registered():
                 SamplingRule(
                     sample_rate=1.0,
                     service="my-service",
-                    name=SamplingRule.NO_RULE,
+                    name=None,
                     resource="*",
                     tags={"care_about": "yes", "region": "us-*"},
                     provenance="customer",
@@ -620,7 +620,9 @@ def test_rc_default_products_registered():
     ],
 )
 def test_trace_sampling_rules_conversion(rc_rules, expected_config_rules, expected_sampling_rules):
-    trace_sampling_rules = config._convert_rc_trace_sampling_rules(rc_rules, None)
+    trace_sampling_rules = _convert_rc_trace_sampling_rules(
+        {"tracing_sampling_rate": None, "tracing_sampling_rules": rc_rules}
+    )
 
     assert trace_sampling_rules == expected_config_rules
     if trace_sampling_rules is not None:
