@@ -5,7 +5,7 @@ def check_pprof_file(
     filename,  # type: str
     sample_type="cpu-samples",
 ):
-    profile = pprof_utils.parse_profile(filename)
+    profile = pprof_utils.parse_newest_profile(filename)
 
     samples = pprof_utils.get_samples_with_value_type(profile, sample_type)
     assert len(samples) >= 1
