@@ -6,6 +6,7 @@ import typing as t
 
 from cassandra.cluster import Cluster
 from cassandra.cluster import NoHostAvailable
+from contrib.config import AZURE_SERVICE_BUS_EMULATOR_CONFIG
 from contrib.config import CASSANDRA_CONFIG
 from contrib.config import ELASTICSEARCH_CONFIG
 from contrib.config import HTTPBIN_CONFIG
@@ -135,6 +136,16 @@ def check_moto(url):
     requests.get(url).raise_for_status()
 
 
+@try_until_timeout(
+    Exception,
+    tries=120,
+    timeout=1,
+    args={"url": "http://{host}:{port}/health".format(**AZURE_SERVICE_BUS_EMULATOR_CONFIG)},
+)
+def check_azureservicebusemulator(url):
+    requests.get(url).raise_for_status()
+
+
 if __name__ == "__main__":
     check_functions = {
         "cassandra": check_cassandra,
@@ -148,6 +159,7 @@ if __name__ == "__main__":
         "rabbitmq": check_rabbitmq,
         "testagent": check_agent,
         "vertica": check_vertica,
+        "azureservicebusemulator": check_azureservicebusemulator,
     }
     if len(sys.argv) >= 2:
         for service in sys.argv[1:]:
